@@ -1,8 +1,8 @@
 from __future__ import annotations
 
-import xdsl.dialects.builtin as builtin
+from typing import Any, Callable, Generic, Literal, TypeAlias, TypeVar
 
-from typing import Any, Callable, Generic, TypeAlias, TypeVar, Literal
+import xdsl.dialects.builtin as builtin
 from xdsl.dialects.builtin import Signedness
 
 
@@ -15,10 +15,14 @@ class _FrontendType:
 
 
 # Type parameters for integers.
-_Width = TypeVar("_Width", bound=int, covariant=True)
-_Signedness = TypeVar("_Signedness", bound=Signedness, covariant=True)
+_Width = TypeVar("_Width", bound=int)
+_Signedness = TypeVar("_Signedness", bound=Signedness)
 
 
+# Note the types ignored below:
+# a) on each function, since the functions are constrained on a limited set of
+#    known types, and _Integer can represent types outside of that set.
+# b) on functions that return `bool` in object, instead of `i1`
 class _Integer(Generic[_Width, _Signedness], _FrontendType):
     """
     Represents an integer type in the frontend. Should not be used explicitly.
@@ -26,73 +30,101 @@ class _Integer(Generic[_Width, _Signedness], _FrontendType):
 
     @staticmethod
     def to_xdsl() -> Callable[..., Any]:
-        return builtin.IntegerType.from_width
+        return builtin.IntegerType
 
     def __add__(
-            self,
-            other: _Integer[_Width,
-                            _Signedness]) -> _Integer[_Width, _Signedness]:
+        self, other: _Integer[_Width, _Signedness]
+    ) -> _Integer[_Width, _Signedness]:
         from xdsl.frontend.dialects.arith import addi
-        return addi(self, other)
+
+        return addi(
+            self,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+            other,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+        )
 
     def __and__(
-            self,
-            other: _Integer[_Width,
-                            _Signedness]) -> _Integer[_Width, _Signedness]:
+        self, other: _Integer[_Width, _Signedness]
+    ) -> _Integer[_Width, _Signedness]:
         from xdsl.frontend.dialects.arith import andi
-        return andi(self, other)
+
+        return andi(
+            self,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+            other,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+        )
 
     def __lshift__(
-            self,
-            other: _Integer[_Width,
-                            _Signedness]) -> _Integer[_Width, _Signedness]:
+        self, other: _Integer[_Width, _Signedness]
+    ) -> _Integer[_Width, _Signedness]:
         from xdsl.frontend.dialects.arith import shli
-        return shli(self, other)
+
+        return shli(
+            self,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+            other,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+        )
 
     def __mul__(
-            self,
-            other: _Integer[_Width,
-                            _Signedness]) -> _Integer[_Width, _Signedness]:
+        self, other: _Integer[_Width, _Signedness]
+    ) -> _Integer[_Width, _Signedness]:
         from xdsl.frontend.dialects.arith import muli
-        return muli(self, other)
+
+        return muli(
+            self,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+            other,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+        )
 
     def __rshift__(
-            self,
-            other: _Integer[_Width,
-                            _Signedness]) -> _Integer[_Width, _Signedness]:
+        self, other: _Integer[_Width, _Signedness]
+    ) -> _Integer[_Width, _Signedness]:
         from xdsl.frontend.dialects.arith import shrsi
-        return shrsi(self, other)
+
+        return shrsi(
+            self,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+            other,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+        )
 
     def __sub__(
-            self,
-            other: _Integer[_Width,
-                            _Signedness]) -> _Integer[_Width, _Signedness]:
+        self, other: _Integer[_Width, _Signedness]
+    ) -> _Integer[_Width, _Signedness]:
         from xdsl.frontend.dialects.arith import subi
-        return subi(self, other)
 
-    def __eq__(self, other: _Integer[_Width, _Signedness]) -> i1:
+        return subi(
+            self,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+            other,  # pyright: ignore[reportGeneralTypeIssues, reportUnknownVariableType]
+        )
+
+    def __eq__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self, other: _Integer[_Width, _Signedness]
+    ) -> i1:
         from xdsl.frontend.dialects.arith import cmpi
-        return cmpi(self, other, "eq")
+
+        return cmpi(self, other, "eq")  # pyright: ignore[reportGeneralTypeIssues]
 
     def __ge__(self, other: _Integer[_Width, _Signedness]) -> i1:
         from xdsl.frontend.dialects.arith import cmpi
-        return cmpi(self, other, "sge")
+
+        return cmpi(self, other, "sge")  # pyright: ignore[reportGeneralTypeIssues]
 
     def __gt__(self, other: _Integer[_Width, _Signedness]) -> i1:
         from xdsl.frontend.dialects.arith import cmpi
-        return cmpi(self, other, "sgt")
+
+        return cmpi(self, other, "sgt")  # pyright: ignore[reportGeneralTypeIssues]
 
     def __le__(self, other: _Integer[_Width, _Signedness]) -> i1:
         from xdsl.frontend.dialects.arith import cmpi
-        return cmpi(self, other, "sle")
+
+        return cmpi(self, other, "sle")  # pyright: ignore[reportGeneralTypeIssues]
 
     def __lt__(self, other: _Integer[_Width, _Signedness]) -> i1:
         from xdsl.frontend.dialects.arith import cmpi
-        return cmpi(self, other, "slt")
 
-    def __ne__(self, other: _Integer[_Width, _Signedness]) -> i1:
+        return cmpi(self, other, "slt")  # pyright: ignore[reportGeneralTypeIssues]
+
+    def __ne__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self, other: _Integer[_Width, _Signedness]
+    ) -> i1:
         from xdsl.frontend.dialects.arith import cmpi
-        return cmpi(self, other, "ne")
+
+        return cmpi(self, other, "ne")  # pyright: ignore[reportGeneralTypeIssues]
 
 
 # Type aliases for signless integers.
@@ -135,14 +167,17 @@ class _Float16(_FrontendType):
 
     def __add__(self, other: f16) -> f16:
         from xdsl.frontend.dialects.arith import addf
+
         return addf(self, other)
 
     def __sub__(self, other: f16) -> f16:
         from xdsl.frontend.dialects.arith import subf
+
         return subf(self, other)
 
     def __mul__(self, other: f16) -> f16:
         from xdsl.frontend.dialects.arith import mulf
+
         return mulf(self, other)
 
 
@@ -158,14 +193,17 @@ class _Float32(_FrontendType):
 
     def __add__(self, other: f32) -> f32:
         from xdsl.frontend.dialects.arith import addf
+
         return addf(self, other)
 
     def __sub__(self, other: f32) -> f32:
         from xdsl.frontend.dialects.arith import subf
+
         return subf(self, other)
 
     def __mul__(self, other: f32) -> f32:
         from xdsl.frontend.dialects.arith import mulf
+
         return mulf(self, other)
 
 
@@ -181,14 +219,17 @@ class _Float64(_FrontendType):
 
     def __add__(self, other: f64) -> f64:
         from xdsl.frontend.dialects.arith import addf
+
         return addf(self, other)
 
     def __sub__(self, other: f64) -> f64:
         from xdsl.frontend.dialects.arith import subf
+
         return subf(self, other)
 
     def __mul__(self, other: f64) -> f64:
         from xdsl.frontend.dialects.arith import mulf
+
         return mulf(self, other)
 
 
