@@ -1,8 +1,10 @@
 from __future__ import annotations
+
 from dataclasses import dataclass, field
 from io import StringIO
+from typing import NoReturn
 
-from xdsl.ir import IRNode, Block, Operation, Region
+from xdsl.ir import Block, IRNode, Operation, Region
 from xdsl.utils.exceptions import DiagnosticException
 
 
@@ -15,23 +17,26 @@ class Diagnostic:
         self.op_messages.setdefault(op, []).append(message)
 
     def raise_exception(
-            self,
-            message: str,
-            ir: IRNode,
-            exception_type: type[Exception] = DiagnosticException) -> None:
+        self,
+        message: str,
+        ir: IRNode,
+        exception_type: type[Exception] = DiagnosticException,
+        underlying_error: Exception | None = None,
+    ) -> NoReturn:
         """Raise an exception, that will also print all messages in the IR."""
         from xdsl.printer import Printer
+
         f = StringIO()
-        p = Printer(stream=f, diagnostic=self)
+        p = Printer(stream=f, diagnostic=self, print_generic_format=True)
         toplevel = ir.get_toplevel_object()
         if isinstance(toplevel, Operation):
             p.print_op(toplevel)
+            print("\n", file=f)
         elif isinstance(toplevel, Block):
             p.print_block(toplevel)
         elif isinstance(toplevel, Region):
-            # TOFIX: Is that ever used. Revisit the whole exception
-            p._print_region(toplevel)  # TOFIX #type: ignore
+            p.print_region(toplevel)
         else:
             assert "xDSL internal error: get_toplevel_object returned unknown construct"
 
-        raise exception_type(message + "\n\n" + f.getvalue())
+        raise exception_type(message + "\n\n" + f.getvalue()) from underlying_error
